"use strict";

let { classes: Cc, interfaces: Ci, results: Cr, utils: Cu } = Components;
Cu.import("resource://gre/modules/XPCOMUtils.jsm");
XPCOMUtils.importRelative(this, "base/arDialog.jsm");
XPCOMUtils.importRelative(this, "base/arPrototype.jsm");
XPCOMUtils.importRelative(this, "base/arTimer.jsm");
XPCOMUtils.importRelative(this, "UnMHTSave.jsm");

let EXPORTED_SYMBOLS = ["UnMHTAdditionalDialogClass"];

/**
 * 追加ファイルダイアログ
 *
 * ウィンドウの引数
 * {
 *   callback: {Function}
 *             終了時のコールバック (必須)
 *               function(result, additionalURIs)
 *                 *param   {number} result
 *                          結果
 *                            UnMHTSave.RESULT_OK
 *                            UnMHTSave.RESULT_CANCEL
 *                 *param   {Array.<string>} additionalURIs
 *                          追加の URI
 * }
 *
 * @param   {nsIDOMWindow} window
 *          対象のウィンドウ
 * @extends    arDialog
 * @implements arIDialog
 */
function UnMHTAdditionalDialogClass(window) {
  arDialog.call(this, window);
}
UnMHTAdditionalDialogClass.prototype = {
  /* ==== arIDialog インターフェース ==== */
  
  /**
   * 破棄
   */
  destruct: function() {
    this._callback = null;
    arDialog.prototype.destruct.call(this);
  },
  
  /**
   * ロードイベント
   */
  onLoad: function() {
    let arg = this._window.arguments[0];
    
    if ("callback" in arg) {
      this._callback = arg.callback;
    }
  },
  
  /**
   * ダイアログを OK したイベント
   *
   * @returns {boolean}
   *          ダイアログを閉じるか
   */
  onDialogAccept: function() {
    if (!this._callback) {
      return true;
    }
    
    let additionalURIs = UnMHTSave.parseAdditionalURIs(this.el("add_uris").value);
    let callback = this._callback;
    
    arTimer.setTimeout(function() {
      callback(UnMHTSave.RESULT_OK, additionalURIs);
    }, 100);
    
    return true;
  },
  
  /**
   * ダイアログをキャンセルしたイベント
   *
   * @returns {boolean}
   *          ダイアログを閉じるか
   */
  onDialogCancel: function() {
    if (!this._callback) {
      return true;
    }
    
    let callback = this._callback;
    arTimer.setTimeout(function() {
      callback(UnMHTSave.RESULT_CANCEL, null);
    }, 100);
    
    return true;
  },
  
  /* ==== private ==== */
  
  _callback: null, /** 終了時コールバック
                    *  @type {Function} */
};
arPrototype.extend(UnMHTAdditionalDialogClass, arDialog);
