"use strict";

let { classes: Cc, interfaces: Ci, results: Cr, utils: Cu } = Components;
Cu.import("resource://gre/modules/Services.jsm");
Cu.import("resource://gre/modules/XPCOMUtils.jsm");

let EXPORTED_SYMBOLS = ["UnMHTDLFHandledLocations"];

/**
 * UnMHTDocumentLoaderFactory がハンドル済みの mht ファイルを管理する
 */
let UnMHTDLFHandledLocations = {
  /**
   * 登録する
   *
   * @param   {string} location
   *          アドレス
   */
  register: function(location) {
    this._handled.add(location);
  },
  
  /**
   * 登録されているか返す
   *
   * @param   {string} location
   *          アドレス
   * @returns {boolean}
   *          登録されているか
   */
  isRegistered: function(location) {
    return this._handled.has(location);
  },
  
  /**
   * 登録解除する
   *
   * @param   {string} location
   *          アドレス
   */
  unregister: function(location) {
    this._handled.delete(location);
  },
  
  /* ==== private ==== */
  
  _handled: new Set()  /** DLF 等によってハンドル済みの mht ファイルの URI
                        *  @type {Set.<string>} */
};
