"use strict";

let { classes: Cc, interfaces: Ci, results: Cr, utils: Cu } = Components;
Cu.import("resource://gre/modules/Services.jsm");
Cu.import("resource://gre/modules/XPCOMUtils.jsm");
XPCOMUtils.importRelative(this, "UnMHTScheme.jsm");
XPCOMUtils.importRelative(this, "UnMHTWindowUtil.jsm");
XPCOMUtils.defineLazyGetter(this, "arFileUtil", function() {
  XPCOMUtils.importRelative(this, "base/arFileUtil.jsm");
  return arFileUtil;
});

let EXPORTED_SYMBOLS = ["UnMHTDocumentCategory"];

/**
 * ドキュメントの情報を取得する
 */
let UnMHTDocumentCategory = {
  /**
   * 保存可能かどうかを調べる
   *
   * @param   {nsIDOMWindow} window
   *          実行中のウィンドウ
   *          null 可
   * @param   {nsIDOMDocument} targetDocument
   *          対象のドキュメント
   * @param   {boolean} allowOtherResource
   *          HTML 以外のファイルを許可するか
   * @returns {boolean}
   *          保存可能かどうか
   */
  isSavable: function(window, targetDocument, allowOtherResource) {
    try {
      /* ==== FIXME: 最近動作チェックしてない ==== */
      if (window &&
          "gIeTab" in window &&
          window.gIeTab.isIeEngine()) {
        return false;
      }
    } catch (e) {
    }
    
    if (window &&
        UnMHTWindowUtil.isMailWindow(window) &&
        targetDocument.location.href == "about:blank") {
      /* 複数のメッセージを選択しているとき等 */
      return false;
    }
    if (window &&
        UnMHTWindowUtil.isMessagePaneCollapsed(window)) {
      return false;
    }
    
    if (targetDocument instanceof Ci.nsIDOMXULDocument) {
      return false;
    }
    
    if ("contentType" in targetDocument &&
        targetDocument.contentType &&
        (/^text/.test(targetDocument.contentType) ||
         /\/x(ht)?ml(\+xml)?$/.test(targetDocument.contentType))) {
      return true;
    }
    
    return allowOtherResource;
  },
  
  /**
   * ローカルの MHT ファイルかどうかを返す
   *
   * @param   {nsIDOMWindow} window
   *          実行中のウィンドウ
   *          null 可
   * @param   {nsIDOMDocument} targetDocument
   *          対象のドキュメント
   * @returns {boolean}
   *          ローカルの MHT ファイルかどうか
   */
  isLocalMHT: function(window, targetDocument) {
    let location;
    try {
      location = targetDocument.location.href;
    } catch (e) {
      /* 対象が生きていない */
      return false;
    }
    if (UnMHTScheme.isUnMHTURI(location)) {
      let param = UnMHTScheme.getUnMHTURIParam(location);
      return ("scheme" in param && param.scheme == "file");
    }
    return false;
  },

  /**
   * ローカルファイルかどうかを返す
   * ローカルファイルであればローカルファイルを格納する
   *
   * @param   {string} location
   *          対象の URI
   * @param   {object} file
   *            {
   *              value: {nsIFile}
   *                     ローカルファイル
   *            }
   * @returns {boolean}
   *          ローカルファイルかどうか
   */
  isLocalFile: function(location, file) {
    if (UnMHTScheme.isUnMHTURI(location)) {
      location = UnMHTScheme.deUnMHTURI(location);
    }
    
    if (!/^file:\/\/\//.test(location)) {
      return false;
    }
    
    file.value = arFileUtil.getFileFromURLSpec(location);
    
    return true;
  },

  /**
   * ドキュメントが生きているかどうか調べる
   *
   * @param   {nsIDOMDocument} targetDocument
   *          対象のドキュメント
   * @returns {boolean}
   *          ドキュメントが生きているか
   */
  isAlive: function(targetDocument) {
    try {
      /* とりあえず適当にドキュメントのプロパティにアクセスする
       * ドキュメントが生きていなければ例外が飛ぶ */
      return targetDocument.location.href != "";
    } catch (e) {
      return false;
    }
  }
};
