"use strict";

let { classes: Cc, interfaces: Ci, results: Cr, utils: Cu } = Components;
Cu.import("resource://gre/modules/Services.jsm");
Cu.import("resource://gre/modules/XPCOMUtils.jsm");
XPCOMUtils.importRelative(this, "base/arFileUtil.jsm");
XPCOMUtils.importRelative(this, "base/arFilePicker.jsm");
XPCOMUtils.importRelative(this, "base/arPrototype.jsm");
XPCOMUtils.importRelative(this, "UnMHTPref.jsm");

let EXPORTED_SYMBOLS = ["UnMHTFilePicker"];

/**
 * ファイル選択ダイアログ
 */
function UnMHTFilePickerClass() {
  arFilePickerClass.call(this);
}
UnMHTFilePickerClass.prototype = {
  /* ==== private ==== */
  
  /**
   * 開くのに最後に使用したディレクトリを取得する
   *
   * @param   {nsIDOMWindow} window
   *          親ウィンドウ
   * @returns {nsIFile}
   *          最後に使用したディレクトリ
   *          無い場合は null
   */
  getLastOpenDir: function(window) {
    if (UnMHTPref.getPref("use_global_lastdir")) {
      return arFilePickerClass.prototype.getLastOpenDir.call(this, window);
    } else {
      let path = UnMHTPref.getPref("open.lastdir");
      if (path) {
        return arFileUtil.getFile(path);
      }
    }
    
    return null;
  },
  
  /**
   * 開くのに最後に使用したディレクトリを設定する
   *
   * @param   {nsIDOMWindow} window
   *          親ウィンドウ
   * @param   {nsIFile} dir
   *          最後に使用したディレクトリ
   */
  setLastOpenDir: function(window, dir) {
    if (UnMHTPref.getPref("use_global_lastdir")) {
      return arFilePickerClass.prototype.setLastOpenDir.call(this, window,
                                                             dir);
    } else {
      UnMHTPref.setPref("open.lastdir", dir.path);
    }
  },
  
  /**
   * 保存するのに最後に使用したディレクトリを取得する
   *
   * @param   {nsIDOMWindow} window
   *          親ウィンドウ
   * @param   {string} uri
   *          保存対象の URI
   *          不要ならば null
   * @returns {nsIFile}
   *          最後に使用したディレクトリ
   *          無い場合は null
   */
  getLastSaveDir: function(window, uri) {
    if (UnMHTPref.getPref("use_global_lastdir")) {
      return arFilePickerClass.prototype.getLastSaveDir.call(this, window,
                                                             uri);
    } else {
      let path = UnMHTPref.getPref("save.lastdir");
      if (path) {
        return arFileUtil.getFile(path);
      }
    }
    
    return null;
  },
  
  /**
   * 保存するのに最後に使用したディレクトリを設定する
   *
   * @param   {nsIDOMWindow} window
   *          親ウィンドウ
   * @param   {string} uri
   *          保存対象の URI
   *          不要ならば null
   * @param   {nsIFile} dir
   *          最後に使用したディレクトリ
   */
  setLastSaveDir: function(window, uri, dir) {
    if (UnMHTPref.getPref("use_global_lastdir")) {
      arFilePickerClass.prototype.setLastSaveDir.call(this, window,
                                                      uri, dir);
    } else {
      UnMHTPref.setPref("save.lastdir", dir.path);
    }
  }
};
arPrototype.extend(UnMHTFilePickerClass, arFilePickerClass);

let UnMHTFilePicker = new UnMHTFilePickerClass();
