"use strict";

let { classes: Cc, interfaces: Ci, results: Cr, utils: Cu } = Components;

let EXPORTED_SYMBOLS = ["UnMHTFormatter"];

/**
 * テキストフォーマッタ
 */
let UnMHTFormatter = {
  /**
   * ファイルサイズを "," で区切る
   *
   * @param   {number} size
   *          ファイルサイズ
   * @returns {string}
   *          "," で区切られたファイルサイズ
   */
  getReadableSize: function(size) {
    let readableSize = size.toString();
    
    if (readableSize.length > 3) {
      readableSize
        = this.getReadableSize(readableSize.substr(0, readableSize.length - 3))
        + "," + readableSize.substr(readableSize.length - 3);
    }
    
    return readableSize;
  },
  
  /**
   * 日時を可読形式に変換する
   *
   * @param   {Date} date
   *          日時
   * @returns {string}
   *          日時の可読形式
   */
  getReadableDate: function(date) {
    return Cc["@mozilla.org/intl/scriptabledateformat;1"]
      .getService(Ci.nsIScriptableDateFormat)
      .FormatDateTime(
        "",
        Ci.nsIScriptableDateFormat.dateFormatLong,
        Ci.nsIScriptableDateFormat.timeFormatSeconds,
        date.getFullYear(),
        date.getMonth() + 1,
        date.getDate(),
        date.getHours(),
        date.getMinutes(),
        date.getSeconds());
  }
};
