"use strict";

let { classes: Cc, interfaces: Ci, results: Cr, utils: Cu } = Components;

let EXPORTED_SYMBOLS = ["UnMHTHTMLUtil"];

/**
 * HTML 操作ユーティリティ
 */
let UnMHTHTMLUtil = {
  /**
   * HTML に使えない文字をエスケープする
   * 
   * @param   {string} text
   *          エスケープする文字列
   * @returns {string}
   *          エスケープした文字列
   */
  escapeEntity: function(text) {
    return (text
            .replace(/&/g, "&amp;")
            .replace(/\"/g, "&quot;")
            .replace(/\'/g, "&#x27;")
            .replace(/</g, "&lt;")
            .replace(/>/g, "&gt;")
            .replace(/\xa0/g, "&#xa0;"));
  },
  
  /**
   * HTML に使えない文字のエスケープを解除する
   * 
   * @param   {string} text
   *          エスケープを解除する文字列
   * @returns {string}
   *          エスケープを解除した文字列
   */
  unescapeEntity: function(text) {
    return (text
            .replace(/&gt;/g, ">")
            .replace(/&lt;/g, "<")
            .replace(/&quot;/g, "\"")
            .replace(/&#x27;/g, "\'")
            .replace(/&#xa0;/g, "\xa0")
            .replace(/&nbsp;/g, "\xa0")
            .replace(/&amp;/g, "&"));
  }
};
