"use strict";

let { classes: Cc, interfaces: Ci, results: Cr, utils: Cu } = Components;
Cu.import("resource://gre/modules/XPCOMUtils.jsm");

let EXPORTED_SYMBOLS = ["UnMHTMIMEParam"];

/**
 * content[RFC2045] や disposition[RFC2183] の値
 *
 * @param   {string} mimetype
 *          MIME-Type (オプショナル)
 */
function UnMHTMIMEParam(mimetype) {
  if (mimetype) {
    let re = mimetype.match(/^([^\/]+)\/([^\/]+)$/);
    if (re) {
      this.type = re[1];
      this.subtype = re[2];
    } else {
      this.type = mimetype;
    }
  }
  this.params = new Map();
}
UnMHTMIMEParam.prototype = {
  type: "",      /** disposition-type[RFC2183]/type[RFC2045]
                  *  @type {string} */
  subtype: "",   /** subtype[RFC2045]
                  *  @type {string} */
  params: null,  /** パラメータのマップ
                  *  @type {Map.<string,string>} */
  
  /**
   * 破棄
   */
  destruct: function() {
    this.type = "";
    this.subtype = "";
    this.params = null;
  },
  
  /**
   * 複製する
   *
   * @returns {UnMHTMIMEParam}
   *          複製
   */
  clone: function() {
    let newParam = new UnMHTMIMEParam();
    
    newParam.type = this.type;
    newParam.subtype = this.subtype;
    newParam.params = new Map(this.params);
    
    return newParam;
  }
};
