"use strict";

let { classes: Cc, interfaces: Ci, results: Cr, utils: Cu } = Components;
Cu.import("resource://gre/modules/XPCOMUtils.jsm");
XPCOMUtils.importRelative(this, "base/arArrayUtil.jsm");
XPCOMUtils.importRelative(this, "UnMHTPseudoID.jsm");

let EXPORTED_SYMBOLS = ["UnMHTMIMEUtil"];

/**
 * MIME ユーティリティ
 */
let UnMHTMIMEUtil = {
  /**
   * boundary[RFC2046] を作成する
   *
   * @returns {string}
   *          バウンダリ文字列
   */
  createUniqueBoundary: function() {
    return ("----=_Part_"
            + UnMHTPseudoID.randHex().toUpperCase()
            + "_"
            + UnMHTPseudoID.randHex().toUpperCase()
            + "."
            + Date.now().toString(16).toUpperCase());
  },
  
  /**
   * 拡張子に対応した MIME-Type を返す
   *
   * @param   {string} ext
   *          拡張子
   * @returns {string}
   *          対応した MIME-Type
   */
  getTypeFromExtension: function(ext) {
    /* システム設定が特殊場合に供えて既知のものは先にチェック */
    
    if (/^[sx]?html?$/i.test(ext)) {
      /* XHTML も text/html として扱う */
      return "text/html";
    }
    
    let knownExts = {
      "css":   "text/css",
      "mht":   "message/rfc822",
      "mhtml": "message/rfc822",
      "eml":   "message/rfc822",
      "js":    "text/javascript",
      "txt":   "text/plain",
      "jpg":   "image/jpeg",
      "jpeg":  "image/jpeg",
      "gif":   "image/gif",
      "png":   "image/png",
      "bmp":   "image/bmp",
      "ico":   "image/x-icon",
      "svg":   "image/svg+xml",
      "swf":   "application/swf",
      "jar":   "application/java-archive",
      "class": "application/octet-stream"
    };
    
    if (ext.toLowerCase() in knownExts) {
      return knownExts[ext.toLowerCase()];
    }
    
    if (ext) {
      /* 未知なのでシステム設定を返す */
      let mime = Cc["@mozilla.org/mime;1"].getService(Ci.nsIMIMEService);
      
      try {
        return mime.getTypeFromExtension(ext);
      } catch (e) {
      }
    }
    
    return "application/octet-stream";
  }
};
