"use strict";

let { classes: Cc, interfaces: Ci, results: Cr, utils: Cu } = Components;

let EXPORTED_SYMBOLS = ["UnMHTMailPrivacy"];

/**
 * メールの個人情報を隠す
 */
let UnMHTMailPrivacy = {
  /**
   * メールの URL を置き換える
   *
   * @param   {string} url
   *          メールの中で使用される URL
   * @param   {string} contentLocation
   *          メールの URL
   * @returns {string}
   *          置換した URL
   */
  hideMailURL: function(url, contentLocation) {
    if (this.isMailURL(url)) {
      url
        = url
        .replace(/(^.+\?.*)&type=[^&]+(.*)/, "$1$2")
        .replace(/^(imap|mailbox|news|nntp|pop|snews):\/\/[^\/]*\//, "")
        .replace(contentLocation
                 .replace(/\?.*/, "")
                 .replace(/^(imap|mailbox|news|nntp|pop|snews):\/\/[^\/]*\//, ""),
                 "file:///mail/attachment/");
      
      if (url == "file:///mail/attachment/") {
        url = "file:///mail/index.html";
      }
    }
    
    return url;
  },
  
  /**
   * メールの URL かどうかを返す
   *
   * @param   {string} url
   *          対象の URL
   * @returns {boolean}
   *          メールの URL かどうか
   */
  isMailURL: function(url) {
    if (/^(imap|mailbox|news|nntp|pop|snews):\/\//.test(url)) {
      return true;
    }
    return false;
  }
};
