"use strict";

let { classes: Cc, interfaces: Ci, results: Cr, utils: Cu } = Components;
Cu.import("resource://gre/modules/Services.jsm");
Cu.import("resource://gre/modules/XPCOMUtils.jsm");
XPCOMUtils.importRelative(this, "UnMHTPref.jsm");

let EXPORTED_SYMBOLS = ["UnMHTMaxRunTime"];

/**
 * スクリプト実行時間管理
 */
let UnMHTMaxRunTime = {
  /**
   * 展開時のスクリプトの実行時間を設定する
   */
  setExtractTime: function() {
    this._pushTime(UnMHTPref.getPref("open.max_run_time"),
                   UnMHTPref.getPref("open.max_run_time.time"));
  },
  
  /**
   * 保存時のスクリプトの実行時間を設定する
   */
  setSaveTime: function() {
    this._pushTime(UnMHTPref.getPref("save.max_run_time"),
                   UnMHTPref.getPref("save.max_run_time.time"));
  },

  /**
   * スクリプトの実行時間を元に戻す
   */
  restoreTime: function() {
    this._popTime();
  },
  
  /* ==== private ==== */
  
  _timeStack: [], /** 設定前の値
                   *  @type {number} */
  _timePref: "dom.max_chrome_script_run_time", /** 実行時間の設定名
                                                *  @type {string} */

  /**
   * スクリプトの実行時間を設定する
   *
   * @param   {boolean} enabled
   *          新しい値を使うか
   *          false の場合現在の値をもう一度設定する
   * @param   {number} value
   *          スクリプトの実行時間
   */
  _pushTime: function(enabled, value) {
    let lastValue = -2;
    
    if (Services.prefs.prefHasUserValue(this._timePref)) {
      lastValue = Services.prefs.getIntPref(this._timePref);
    }
    
    this._timeStack.push(lastValue);
    if (!enabled) {
      value = lastValue;
    }
    
    Services.prefs.setIntPref(this._timePref, value);
  },
  
  /**
   * スクリプトの実行時間を元に戻す
   */
  _popTime: function() {
    let lastValue = -2;
    if (this._timeStack.length) {
      lastValue = this._timeStack.pop();
    }
    
    if (lastValue == -2) {
      Services.prefs.clearUserPref(this._timePref);
    } else {
      Services.prefs.setIntPref(this._timePref, lastValue);
    }
  }
};
