"use strict";

let { classes: Cc, interfaces: Ci, results: Cr, utils: Cu } = Components;
Cu.import("resource://gre/modules/Services.jsm");
Cu.import("resource://gre/modules/XPCOMUtils.jsm");

let EXPORTED_SYMBOLS = ["UnMHTMhtmlProtocolHandler"];

/**
 * mhtml プロトコルを通常の URI に置き換える
 *
 * @implements nsIProtocolHandler
 */
function UnMHTMhtmlProtocolHandler() {
}
UnMHTMhtmlProtocolHandler.prototype = {
  classID:          Components.ID("{6a0b6c61-7f15-4b42-af62-da7cfcdc04a8}"),
  contractID:       "@mozilla.org/network/protocol;1?name=mhtml",
  classDescription: "UnMHT Mhtml Protocol Handler JS Component",
  QueryInterface:   XPCOMUtils.generateQI([Ci.nsIProtocolHandler]),
  
  /* ==== nsIProtocolHandler インターフェース ==== */
  
  scheme: "mhtml", /** プロトコルスキーム
                    *  @type {string} */
  defaultPort: -1, /** ポート番号
                    *  @type {number}
                    *  ネットワークベースではないので -1 */
  
  /** プロトコルの詳細
   *  @type {number}
   *  file と同様のプロトコル */
  protocolFlags: (Ci.nsIProtocolHandler.URI_NOAUTH
                  | Ci.nsIProtocolHandler.URI_LOADABLE_BY_ANYONE),
  
  /**
   * ブラックリストのポートを上書きするか
   *
   * @param   {number} port
   *          ポート番号
   * @param   {string} scheme
   *          プロトコルスキーム
   * @returns {boolean}
   *          上書きするか
   */
  allowPort: function(port, scheme) {
    return false;
  },
  
  /**
   * URI を作成する
   *
   * @param   {string} spec
   *          対象の URI
   * @param   {string} charset
   *          対象の文字コード
   * @param   {nsIURI} baseURI
   *          読み込み元の URI
   * @returns {nsIURI}
   *          作成した URI
   */
  newURI: function(spec, charset, baseURI) {
    /* mhtml:http://hoge
     * mhtml://http://hoge
     *   to
     * http://hoge
     */
    if (/^mhtml:(\/\/)?([^\/]+):\/\//.test(spec)) {
      return Services.io.newURI(spec.replace(/^mhtml:(\/\/)?([^\/]+):\/\//,
                                             "$2://"),
                                null, null);
    } else {
      return Services.io.newURI("data:text/plain,", null, null);
    }
  },
  
  /**
   * チャネルを作成する
   *
   * @param   {nsIURI} uri
   *          対象の URI
   * @returns {nsIChannel}
   *          作成したチャネル
   */
  newChannel: function(uri) {
    return Services.io.newChannel("data:text/plain,", null, null);
  }
};
