"use strict";

let { classes: Cc, interfaces: Ci, results: Cr, utils: Cu } = Components;
Cu.import("resource://gre/modules/XPCOMUtils.jsm");
XPCOMUtils.importRelative(this, "UnMHT.jsm");
XPCOMUtils.importRelative(this, "UnMHTInfoPanel.jsm");
XPCOMUtils.importRelative(this, "UnMHTMenu.jsm");
XPCOMUtils.importRelative(this, "UnMHTStringBundle.jsm");

let EXPORTED_SYMBOLS = ["UnMHTOverlay"];

/**
 * メニューの項目を作成する
 * 3 箇所で同じ内容のメニューを使用するのでまとめる
 *
 * @param   {boolean} isMailer
 *          メーラか
 * @param   {nsIXULWindow} window
 *          対象のウィンドウ
 * @param   {arOverly} overlay
 *          オーバーレイ
 * @returns {arOverlayMenuPopupElement} menu
 *          メニュー
 * @param   {string} name
 *          メニューの名前
 */
function createMenuItems(isMailer, window, overlay,
                         menu, name) {
  menu.addItem({
    id: "unmht-" + name + "-preferences",
    label: UnMHTStringBundle.get("menuitem.pref")
  }, function(event) {
    UnMHT.showPrefDialog(window);
  });
  menu.addSeparator({
    id: "unmht-" + name + "-menuitem-separator-save"
  });
  menu.addItem({
    id: "unmht-" + name + "-menuitem-save",
    label: UnMHTStringBundle.get("menuitem.save.single")
  }, function(event) {
    UnMHT.save(window);
  });
  
  if (!isMailer) {
    menu.addItem({
      id: "unmht-" + name + "-menuitem-overwrite",
      label: UnMHTStringBundle.get("menuitem.save.overwrite")
    }, function(event) {
      UnMHT.overwrite(window);
    });
  }
  
  menu.addItem({
    id: "unmht-" + name + "-menuitem-save-selection",
    label: UnMHTStringBundle.get("menuitem.save.selection")
  }, function(event) {
    UnMHT.saveSelection(window);
  });
  
  if (!isMailer) {
    menu.addItem({
      id: "unmht-" + name + "-menuitem-sendmail",
      label: UnMHTStringBundle.get("menuitem.save.mail")
    }, function(event) {
      UnMHT.sendmail(window);
    });
    menu.addItem({
      id: "unmht-" + name + "-menuitem-save-multi",
      label: UnMHTStringBundle.get("menuitem.save.multi")
    }, function(event) {
      UnMHT.savemulti(window);
    });
    menu.addSeparator({
      id: "unmht-" + name + "-menuitem-separator-cache"
    });
    menu.addItem({
      id: "unmht-" + name + "-menuitem-cache-show",
      label: UnMHTStringBundle.get("menuitem.cache.show")
    }, function(event) {
      UnMHT.openCache(window);
    });
    menu.addItem({
      id: "unmht-" + name + "-menuitem-cache-clear",
      label: UnMHTStringBundle.get("menuitem.cache.clear")
    }, function(event) {
      UnMHT.clearCache(window);
    });
    menu.addSeparator({
      id: "unmht-" + name + "-menuitem-separator-extract"
    });
    menu.addItem({
      id: "unmht-" + name + "-menuitem-extract",
      label: UnMHTStringBundle.get("menuitem.extract")
    }, function(event) {
      UnMHT.openWithExtract(window);
    });
    menu.addItem({
      id: "unmht-" + name + "-menuitem-unextract",
      label: UnMHTStringBundle.get("menuitem.show_source")
    }, function(event) {
      UnMHT.openWithoutExtract(window);
    });
    menu.addSeparator({
      NAME: "menuseparator",
      id: "unmht-" + name + "-menuitem-separator-website"
    });
    menu.addItem({
      id: "unmht-" + name + "-menuitem-openwebsite",
      label: UnMHTStringBundle.get("menuitem.openwebsite")
    }, function(event) {
      UnMHT.openWebsite(window);
    });
  }
}

/**
 * プログレスメーターの要素
 * 2 箇所で同じ内容を使用するのでまとめる
 *
 * @param   {boolean} isMailer
 *          メーラか
 * @param   {nsIXULWindow} window
 *          対象のウィンドウ
 * @param   {arOverly} overlay
 *          オーバーレイ
 * @returns {arOverlayElement} progress
 *          プログレスメーター
 */
function createProgressMeterNodes(isMailer, window, overlay,
                                  progress) {
  progress.addElement("progressmeter", {
    id: "unmht-toolbarbutton-progress-meter",
    "class": "progressmeter-statusbar",
    style: "width: 60px; min-width: 60px",
    mode: "determined",
    value: "0"
  });
  progress.addLabel({
    id: "unmht-toolbarbutton-progress-label",
    value: ""
  });
}

/**
 * UI のオーバーレイ生成関数
 *
 * @param   {boolean} isMailer
 *          メーラか
 * @param   {nsIXULWindow} window
 *          対象のウィンドウ
 * @param   {arOverly} overlay
 *          オーバーレイ
 */
function UnMHTOverlay(isMailer, window, overlay) {
  overlay.loadCSS("chrome://unmht/content/ui.css");
  
  overlay.fileMenu.addEventListener("popupshowing", function(event) {
    if (UnMHTMenuMap.has(window)) {
      UnMHTMenuMap.get(window).onFileMenuShowing(event);
    }
  });
  
  overlay.fileMenu.addItem({
    id: "unmht-filemenu-menuitem-save",
    insertafter: "menu_sendLink,menu_saveAs",
    label: UnMHTStringBundle.get("menuitem.save.single")
  }, function(event) {
    UnMHT.save(window);
  });
  
  if (!isMailer) {
    overlay.fileMenu.addItem({
      id: "unmht-filemenu-menuitem-overwrite",
      insertafter: "unmht-filemenu-menuitem-save",
      label: UnMHTStringBundle.get("menuitem.save.overwrite"),
    }, function(event) {
      UnMHT.overwrite(window);
    });
    overlay.fileMenu.addItem({
      id: "unmht-filemenu-menuitem-save-multi",
      insertafter: "unmht-filemenu-menuitem-overwrite",
      label: UnMHTStringBundle.get("menuitem.save.multi"),
    }, function(event) {
      UnMHT.savemulti(window);
    });
  }
  
  let menu = overlay.toolsMenu.addMenu({
    id: "unmht-tools-preferences",
    insertbefore: "webDeveloperMenu,navBeginGlobalItems,javascriptConsole",
    "class": "menu-iconic",
    image: "resource://unmht/images/icon_small.png",
    label: UnMHTStringBundle.get("name"),
  }).addEventListener("popupshowing", function(event) {
    if (UnMHTMenuMap.has(window)) {
      UnMHTMenuMap.get(window).onToolMenuShowing(event);
    }
  });
  createMenuItems(isMailer, window, overlay,
                  menu, "toolmenu");
  
  if (!isMailer) {
    overlay.contentMenu.addEventListener("popupshowing", function(event) {
      if (UnMHTMenuMap.has(window)) {
        UnMHTMenuMap.get(window).onContextMenuShowing(event);
      }
    });
    overlay.contentMenu.addItem({
      id: "unmht-context-menuitem-save",
      insertafter: "context-savepage",
      label: UnMHTStringBundle.get("menuitem.save.single")
    }, function(event) {
      UnMHT.save(window);
    });
    overlay.contentMenu.addItem({
      id: "unmht-context-menuitem-overwrite",
      insertafter: "unmht-context-menuitem-save",
      label: UnMHTStringBundle.get("menuitem.save.overwrite")
    }, function(event) {
      UnMHT.overwrite(window);
    });
    overlay.contentMenu.addItem({
      id: "unmht-context-menuitem-sendmail",
      insertafter: "unmht-context-menuitem-overwrite",
      label: UnMHTStringBundle.get("menuitem.save.mail")
    }, function(event) {
      UnMHT.sendmail(window);
    });
    overlay.contentMenu.addItem({
      id: "unmht-context-menuitem-save-selection",
      insertafter: "unmht-context-menuitem-sendmail",
      label: UnMHTStringBundle.get("menuitem.save.selection")
    }, function(event) {
      UnMHT.saveSelectionInDocument(window, window.document.popupNode.ownerDocument);
    });
    overlay.contentMenu.addItem({
      id: "unmht-context-menuitem-save-link",
      insertafter: "unmht-context-menuitem-save-selection",
      label: UnMHTStringBundle.get("menuitem.save.link")
    }, function(event) {
      UnMHT.saveLink(window);
    });
    overlay.contentMenu.addItem({
      id: "unmht-context-menuitem-quicksave",
      insertafter: "unmht-context-menuitem-save-link",
      label: UnMHTStringBundle.get("menuitem.save.quick")
    }, function(event) {
      UnMHT.quicksave(window);
    });
    
    overlay.frameContentMenu.addItem({
      id: "unmht-frame-context-menuitem-save",
      insertafter: "context-saveframe",
      label: UnMHTStringBundle.get("menuitem.save.single.frame")
    }, function(event) {
      UnMHT.saveDocument(window, window.document.popupNode.ownerDocument);
    });
    overlay.frameContentMenu.addItem({
      id: "unmht-frame-context-menuitem-sendmail",
      insertafter: "unmht-frame-context-menuitem-save",
      label: UnMHTStringBundle.get("menuitem.save.mail.frame")
    }, function(event) {
      UnMHT.sendmailDocument(window, window.document.popupNode.ownerDocument);
    });
  }
  
  let button = overlay.toolbar.addButton({
    id: "unmht-toolbarbutton-preferences",
    type: "menu-button",
    "class": (isMailer
              ? "toolbarbutton-1 chromeclass-toolbar-additional"
              : "toolbarbutton-1"),
    style: "list-style-image: url('resource://unmht/images/icon_middle.png')",
    context: "unmht-toolbar2-popup",
    label: UnMHTStringBundle.get("name"),
    tooltiptext: UnMHTStringBundle.get("name")
  }, function(event) {
    if (("button" in event && event.button != 0) ||
        event.ctrlKey || event.shiftKey ||
        event.altKey || event.metaKey) {
      /* コンテキストメニュー呼び出し */
      return;
    }
    
    if (event.target &&
        event.target.nodeName &&
        /menuitem/i.test(event.target.nodeName)) {
      /* ツールバーのボタン内のメニュー */
      return;
    }
    
    UnMHT.showPrefDialog(window);
  });
  
  menu = button.addMenuPopup();
  menu.addEventListener("popupshowing", function(event) {
    if (UnMHTMenuMap.has(window)) {
      UnMHTMenuMap.get(window).onToolbarMenuShowing(event);
    }
  });
  createMenuItems(isMailer, window, overlay,
                  menu, "toolbar");
  
  overlay.toolbar.addButton({
    id: "unmht-toolbarbutton-save",
    "class": (isMailer
              ? "toolbarbutton-1 chromeclass-toolbar-additional"
              : "toolbarbutton-1"),
    style: "list-style-image: url('resource://unmht/images/save_middle.png')",
    label: UnMHTStringBundle.get("menuitem.save.single"),
    tooltiptext: UnMHTStringBundle.get("menuitem.save.single")
  }, function(event) {
    UnMHT.save(window);
  });
  overlay.toolbar.addButton({
    id: "unmht-toolbarbutton-quicksave",
    "class": (isMailer
              ? "toolbarbutton-1 chromeclass-toolbar-additional"
              : "toolbarbutton-1"),
    style: "list-style-image: url('resource://unmht/images/quicksave_middle.png')",
    label: UnMHTStringBundle.get("menuitem.save.quick"),
    tooltiptext: UnMHTStringBundle.get("menuitem.save.quick")
  }, function(event) {
    UnMHT.quicksave(window);
  });
  if (!isMailer) {
    overlay.toolbar.addButton({
      id: "unmht-toolbarbutton-savemulti",
      "class": "toolbarbutton-1",
      style: "list-style-image: url('resource://unmht/images/savemulti_middle.png')",
      label: UnMHTStringBundle.get("menuitem.save.multi"),
      tooltiptext: UnMHTStringBundle.get("menuitem.save.multi")
    }, function(event) {
      UnMHT.savemulti(window);
    });
  }
  
  if (!isMailer) {
    let progress = overlay.toolbar.addItem({
      id: "unmht-toolbarbutton-progress",
      context: "unmht-progress-popup",
      label: UnMHTStringBundle.get("toolbar.progress"),
      tooltiptext: UnMHTStringBundle.get("toolbar.progress"),
      align: "center",
      pack: "center",
    });
    createProgressMeterNodes(isMailer, window, overlay,
                             progress);
  } else {
    let progress = overlay.statusBar.addPanel({
      id: "unmht-toolbarbutton-progress",
      "class": "statusbarpanel-iconic",
      context: "unmht-progress-popup",
      label: UnMHTStringBundle.get("toolbar.progress"),
      tooltiptext: UnMHTStringBundle.get("toolbar.progress"),
      align: "center",
      pack: "center"
    });
    createProgressMeterNodes(isMailer, window, overlay,
                             progress);
  }
  
  if (!isMailer) {
    let infoPanel = overlay.mainPopupSet.addPanel({
      id: "unmht-info-panel",
      type: "arrow",
      orient: "vertical",
      ignorekeys: "true",
      hidden: "true"
    });
    let grid = infoPanel.addGrid(null, {
      columns: [{}, { flex: 1}]
    });
    let row;
    row = grid.addRow({
      align: "center",
    });
    row.addLabel({
      value: UnMHTStringBundle.get("info.original_address")
    });
    row.addTextBox({
      id: "unmht-info-panel-original",
      value: "",
      flex: "1",
      readonly: "true"
    });
    row.addButton({
      label: UnMHTStringBundle.get("info.copy")
    }).addEventListener("click", function(event) {
      if (UnMHTInfoPanelMap.has(window)) {
        UnMHTInfoPanelMap.get(window).onInfoCopyClick(event);
      }
    });
    
    row = grid.addRow({
      id: "unmht-info-panel-row-saved-from",
      align: "center"
    });
    row.addLabel({
      value: UnMHTStringBundle.get("info.saved_from")
    });
    row.addLabel({
      id: "unmht-info-panel-saved-from",
      value: "",
      crop: "end",
      flex: "1"
    }).addEventListener("click", function(event) {
      if (UnMHTInfoPanelMap.has(window)) {
        UnMHTInfoPanelMap.get(window).onInfoSavedFromClick(event);
      }
    });
    
    row = grid.addRow({
      id: "unmht-info-panel-row-saved-date",
      align: "center",
    });
    row.addLabel({
      value: UnMHTStringBundle.get("info.saved_date")
    });
    row.addLabel({
      id: "unmht-info-panel-saved-date",
      value: "",
      flex: "1"
    });
  }
  
  menu = overlay.mainPopupSet.addMenuPopup({
    id: "unmht-toolbar2-popup"
  }).addEventListener("popupshowing", function(event) {
    if (UnMHTMenuMap.has(window)) {
      UnMHTMenuMap.get(window).onToolbarMenu2Showing(event);
    }
  });
  createMenuItems(isMailer, window, overlay,
                  menu, "toolbar2");
  
  menu = overlay.mainPopupSet.addMenuPopup({
    id: "unmht-progress-popup",
    position: "after_start"
  });
  menu.addItem({
    id: "unmht-progress-popup-cancel",
    label: UnMHTStringBundle.get("menuitem.save.cancel")
  }, function(event) {
    UnMHT.cancelSave(window);
  });
  
  if (!isMailer) {
    overlay.URLBarIcons.addIcon({
      POSITION: "first",
      id: "unmht-urlbar-icon",
      "class": "urlbar-icon"
    }).addEventListener("click", function(event) {
      if (UnMHTInfoPanelMap.has(window)) {
        UnMHTInfoPanelMap.get(window).onInfoIconClick(event);
      }
    });
  }
}
