"use strict";

let { classes: Cc, interfaces: Ci, results: Cr, utils: Cu } = Components;
Cu.import("resource://gre/modules/Services.jsm");
Cu.import("resource://gre/modules/XPCOMUtils.jsm");
XPCOMUtils.importRelative(this, "base/arAppInfo.jsm");
XPCOMUtils.importRelative(this, "base/arPrefClass.jsm");
XPCOMUtils.importRelative(this, "base/arPrototype.jsm");
XPCOMUtils.importRelative(this, "base/arUconv.jsm");
XPCOMUtils.importRelative(this, "UnMHTVersion.jsm");
XPCOMUtils.defineLazyGetter(this, "arFileUtil", function() {
  XPCOMUtils.importRelative(this, "base/arFileUtil.jsm");
  return arFileUtil;
});

let EXPORTED_SYMBOLS = ["UnMHTPref"];

/**
 * メールの引数
 * @type {string}
 * import がループしないためにメールの引数はここで宣言する */
let _thunderbirdArg
= "-remote \"xfeDoCommand(composeMessage,subject=\'%TITLE%\',attachment=\'%URI%\')\"";
let _seamonkeyArg
= "-remote \"xfeDoCommand(composeMessage,subject=\'%TITLE_NOSP%\',attachment=\'%URI%\')\"";
let _macmailArg
= "\"%TITLE%\" \"%FILE%\"";

/**
 * デフォルトのメーラ
 * @type {string}
 */
let defaultMail = arAppInfo.isSeaMonkey ? "seamonkey" : "thunderbird";

/**
 * 設定操作と既定値
 *
 * @extends arPrefClass
 */
function UnMHTPrefClass() {
  arPrefClass.call(this);
  
  let defPrefs = {
    /* General */
    "version": "",
    
    "open.multiple.action": "tabs", /* tabs/index/first */
    
    "open.ppt.skip_warning": false,
    
    "open.offline": false,
    
    "statusbar.icon": false,
    
    /* Save */
    "save.contents.video": "ask", /* save/ignore/ask */
    "save.contents.embed": "ask", /* save/ignore/ask */

    "save.header.mail.original_field": false,
    "save.header.mail.show": false,
    
    "save.insert.link": false,
    "save.insert.link.pos": "bottom", /* top/bottom */
    "save.insert.link.date": true,
    "save.insert.link.version": true,
    "save.insert.link.nolocal": false,
    
    "save.history.register": true,
    "save.progress.statusbar": false,
    "save.show_errors": true,
    "save.show_option_dialog": false,
    
    "save.local.same_dir": false,
    "save.local.same_dir.append": "",
    
    "save.single.state": "current", /* current/original */
    "save.single.state.remember": true,
    "save.single.add": false,
    
    /* Multiple */
    "save.multi.state": "current", /* current/original */
    "save.multi.state.remember": true,
    "save.multi.format": "separated", /* separated/single
                                       * /single_noindex */
    
    "save.multi.dir": "",
    
    "save.multi.dup.action": "rename", /* replace/rename/ask */
    
    "save.multi.autoclose": false,
    "save.multi.tabs.autoclose": false,
    "save.multi.check_none": false,

    "save.multi.index.style": false,
    "save.multi.index.style.content": "",
    
    /* QuickSave */
    "save.quick.state": "current", /* current/original */
    
    "save.quick.dir": "",
    
    "save.quick.dup.action": "rename", /* replace/rename/ask */
    
    "save.quick.use_selection": false,
    
    /* Mail */
    "save.mail.state": "current", /* current/original */
    "save.mail.state.remember": true,
    
    "save.mail.app.type": defaultMail, /* thunderbird/seamonkey
                                        * /macmail/other */
    "save.mail.app": "",
    
    "save.mail.arg.type": defaultMail, /* thunderbird/seamonkey
                                        * /macmail/other */
    "save.mail.arg.other": toUTF8(_thunderbirdArg),

    "save.mail.zip": false,
    
    /* Filename */
    "save.filename": "title", /* leafname/title/custom */
    
    "save.filename.dup.rename.digits": 1,
    "save.filename.dup.rename.init": 2,
    
    "save.filename.custom": "%FILENAME%",
    "save.filename.custom.am": "a.m.",
    "save.filename.custom.pm": "p.m.",
    "save.filename.custom.limit": false,
    "save.filename.custom.limit.count": 100,
    "save.filename.custom.mail_date": false,

    "save.filename.replace.list": "[]",
    
    /* Menu */
    "menu.context.save": true,
    "menu.context.save.quick": true,
    "menu.context.save.mail": true,
    "menu.tool": true,
    
    /* Keyboard */
    "shortcut.save.single": false,
    "shortcut.save.single.keycode": "VK_S",
    "shortcut.save.single.mods.alt": true,
    "shortcut.save.single.mods.ctrl": true,
    "shortcut.save.single.mods.meta": false,
    "shortcut.save.single.mods.shift": false,
    
    "shortcut.save.multi": false,
    "shortcut.save.multi.keycode": "VK_A",
    "shortcut.save.multi.mods.alt": true,
    "shortcut.save.multi.mods.ctrl": true,
    "shortcut.save.multi.mods.meta": false,
    "shortcut.save.multi.mods.shift": false,
    
    "shortcut.save.quick": false,
    "shortcut.save.quick.keycode": "VK_Q",
    "shortcut.save.quick.mods.alt": true,
    "shortcut.save.quick.mods.ctrl": true,
    "shortcut.save.quick.mods.meta": false,
    "shortcut.save.quick.mods.shift": false,

    "shortcut.save.overwrite": false,
    "shortcut.save.overwrite.keycode": "VK_E",
    "shortcut.save.overwrite.mods.alt": true,
    "shortcut.save.overwrite.mods.ctrl": true,
    "shortcut.save.overwrite.mods.meta": false,
    "shortcut.save.overwrite.mods.shift": false,
    
    /* Convert */
    "convert.dir.suffix": "_files",
    "convert.dir.sub": false,
    
    /* Advanced */
    "open.max_run_time": true,
    "open.max_run_time.time": 0,
    
    "save.max_run_time": true,
    "save.max_run_time.time": 0,
    
    "menu.all.extract": false,
    "menu.all.cache": false,
    
    "open.cache.expire": true,
    "open.cache.expire.time": 30,
    
    "save.insert.url": false,
    "save.import.limit.exceeded.action": "ask", /* save/ignore/ask */
    
    "save.mail.debug": false,
    
    "use_global_lastdir": true,
    
    "save.filename.replace.ie_bug.disable": false,
    
    /* Other */
    
    "open.lastdir": "",
    "save.lastdir": "",
    
    /* Hidden */
    
    "integrataion.maf": true,
    "integrataion.scrapbook": true,
    
    "protocol.mhtml": true,
    
    "open.extensions": "[\"mht\",\"mhtml\",\"eml\",\"mbs\"]",
    "open.mime.types": "[\"message/rfc822\",\"message/x-mht\",\"application/mht\",\"application/x-mht\",\"application/mime\",\"application/x-mime\",\"application/x-mimearchive\",\"application/vnd.sealed.mht\",\"multipart/related\",\"text/x-mht\"]",
    "open.mime.extensions": "[[\"mhtml\",\"application/x-mht\"],[\"mht\",\"application/x-mht\"],[\"mbs\",\"application/mbox\"],[\"eml\",\"message/rfc822\"]]",
  };
  this._setDefPrefs(defPrefs);
}
UnMHTPrefClass.prototype = {
  /**
   * メールの引数
   * @type {string}
   */
  thunderbirdArg: _thunderbirdArg,
  seamonkeyArg:   _seamonkeyArg,
  macmailArg:     _macmailArg,
  
  prefix: "extensions.unmht.", /** 設定のプレフィックス
                                *  @type {string} */
  
  /**
   * JSON で格納している設定名のリスト
   * ファイルにエクスポートする際に生データを出力する
   * @type {Array.<string>}
   */
  get JSONDataNames() {
    return [
      "save.filename.replace.list"
    ];
  },
  
  /**
   * 古い設定からのマイグレート
   */
  migrate: function() {
    let version = UnMHTPref.getPref("version");
    let versionComparator = Cc["@mozilla.org/xpcom/version-comparator;1"]
      .getService(Ci.nsIVersionComparator);
    let minVersion = "6.4.0b1";
    if (versionComparator.compare(version, minVersion) >= 0) {
      /* 6.4.0b1 未満では extensions.unmht.version のユーザ設定として
       * 正しいバージョン番号が格納されていないので、これより新しい場合のみ
       * バージョン番号によって各マイグレートの分岐をする */
      return true;
    }
    
    let result = true;
    
    let prefHandler5 = {
      prefHas: function(name) {
        return Services.prefs.prefHasUserValue(name);
      },
      getBoolPref: function(name) {
        return Services.prefs.getBoolPref(name);
      },
      getCharPref: function(name) {
        return unescape(Services.prefs.getCharPref(name));
      },
      getIntPref: function(name) {
        return Services.prefs.getIntPref(name);
      },
      setBoolPref: function(name, value) {
        Services.prefs.setBoolPref(name, value);
      },
      setCharPref: function(name, value) {
        Services.prefs.setCharPref(name, toUTF8(value));
      },
      setIntPref: function(name, value) {
        Services.prefs.setIntPref(name, value);
      }
    };
    
    let prefHandler6 = {
      prefHas: function(name) {
        return Services.prefs.prefHasUserValue(name);
      },
      getBoolPref: function(name) {
        return Services.prefs.getBoolPref(name);
      },
      getCharPref: function(name) {
        return fromUTF8(Services.prefs.getCharPref(name));
      },
      getIntPref: function(name) {
        return Services.prefs.getIntPref(name);
      },
      setBoolPref: function(name, value) {
        Services.prefs.setBoolPref(name, value);
      },
      setCharPref: function(name, value) {
        Services.prefs.setCharPref(name, toUTF8(value));
      },
      setIntPref: function(name, value) {
        Services.prefs.setIntPref(name, value);
      }
    };
    
    try {
      this._migrate_5_7_14_to_6_0_0(prefHandler5);
    } catch (e) {
      result = false;
      Cu.reportError("Failed to migrate.\n"
                     + e + "\n"
                     + e.stack);
    }
    
    try {
      this._migrate_6_0_0_to_6_1_0(prefHandler6);
    } catch (e) {
      result = false;
      Cu.reportError("Failed to migrate.\n"
                     + e + "\n"
                     + e.stack);
    }
    
    try {
      this._migrate_6_3_0_to_6_4_0(prefHandler6);
    } catch (e) {
      result = false;
      Cu.reportError("Failed to migrate.\n"
                     + e + "\n"
                     + e.stack);
    }
    
    return result;
  },
  
  /**
   * ファイルからの設定の読み込みを開始する
   *
   * @param   {nsIFile} file
   *          読み込む対象のファイル
   *          null にすると既定値を書いたファイル扱い
   * @returns {UnMHTPrefFileInputObject}
   *          設定入力オブジェクト
   *          ファイルが不正な場合は null
   *            実体は Map.<string,(boolean|string|number)>
   */
  initPrefFromFile: function(file) {
    let map = arPrefClass.prototype.initPrefFromFile.call(this, file);
    
    let prefHandler6 = {
      map: null,
      
      prefHas: function(name) {
        name = name.replace(/^extensions\.unmht\./, "");
        return this.map.has(name);
      },
      getBoolPref: function(name) {
        name = name.replace(/^extensions\.unmht\./, "");
        return this.map.get(name);
      },
      getCharPref: function(name) {
        name = name.replace(/^extensions\.unmht\./, "");
        return this.map.get(name);
      },
      getIntPref: function(name) {
        name = name.replace(/^extensions\.unmht\./, "");
        return this.map.get(name);
      },
      setBoolPref: function(name, value) {
        name = name.replace(/^extensions\.unmht\./, "");
        this.map.set(name, value);
      },
      setCharPref: function(name, value) {
        name = name.replace(/^extensions\.unmht\./, "");
        this.map.set(name, value);
      },
      setIntPref: function(name, value) {
        name = name.replace(/^extensions\.unmht\./, "");
        this.map.set(name, value);
      }
    };
    
    if (map) {
      prefHandler6.map = map;
      this._migrate_6_3_0_to_6_4_0(prefHandler6);
      return map;
    }
    
    /* Ver.6.2.x 以前のファイル形式 */
    
    map = new Map();
    
    let text = arFileUtil.readData(file).replace(/\r\n/g, "\n");
    
    let ok = false;
    
    let obsTypes = {
      "save.multi.type": "string",
      "save.quick.type": "string",
      "save.mail.type":  "string",
      "save.single.type": "string",
      "save.deep_import.action": "string",
    };
    
    text
      .replace(/extensions\.unmht\.([^\n]+),([^\n]+)\n?/g,
               function(matched, name, value) {
                 name = unescape(name);
                 value = unescape(value);
                 
                 if (name in this._defPrefs) {
                   switch (this.typeof(name)) {
                     case "boolean": map.set(name, value == "true");     break;
                     case "string":  map.set(name, unescape(value));     break;
                     case "number":  map.set(name, parseInt(value, 10)); break;
                   }
                   ok = true;
                 } else if (name in obsTypes) {
                   switch (obsTypes[name]) {
                     case "boolean": map.set(name, value == "true");     break;
                     case "string":  map.set(name, unescape(value));     break;
                     case "number":  map.set(name, parseInt(value, 10)); break;
                   }
                   ok = true;
                 }
                 
                 return "";
               }.bind(this));
    if (!ok) {
      /* Ver.5.x.x 以前のファイル */
      
      let oldMap = new Map();
      
      text
        .replace(/(unmht\.[^\n]+),([^\n]+)\n?/g,
                 function(matched, name, value) {
                   name = unescape(name);
                   value = unescape(value);
                   
                   oldMap.set(name, value);
                   ok = true;
                   
                   return "";
                 });
      if (!ok) {
        return null;
      }
      
      let prefHandler5 = {
        prefHas: function(name) {
          return oldMap.has(name);
        },
        getBoolPref: function(name) {
          return (oldMap.get(name) == "true");
        },
        getCharPref: function(name) {
          return unescape(oldMap.get(name));
        },
        getIntPref: function(name) {
          return parseInt(oldMap.get(name), 10);
        },
        setBoolPref: function(name, value) {
          name = name.replace(/^extensions\.unmht\./, "");
          map.set(name, value);
        },
        setCharPref: function(name, value) {
          name = name.replace(/^extensions\.unmht\./, "");
          map.set(name, value);
        },
        setIntPref: function(name, value) {
          name = name.replace(/^extensions\.unmht\./, "");
          map.set(name, value);
        }
      };
      
      this._migrate_5_7_14_to_6_0_0(prefHandler5);
    }
    
    prefHandler6.map = map;
    this._migrate_6_0_0_to_6_1_0(prefHandler6);
    
    prefHandler6.map = map;
    this._migrate_6_3_0_to_6_4_0(prefHandler6);
    
    return map;
  },
  
  /* ==== private ==== */
  
  /**
   * 5.7.14 から 6.0.0 へのマイグレート
   *
   * @param   {object} prefHandler
   *          設定操作関数セット
   */
  _migrate_5_7_14_to_6_0_0: function(prefHandler) {
    let oldPrefix = "unmht.";
    let newPrefix = this.prefix;
    
    if (!(prefHandler.prefHas(oldPrefix + "version")
          && !prefHandler.prefHas(newPrefix + "version"))) {
      return;
    }
    
    let migrates = [
      ["open.multiple.action", "open_multiple", "string"],
      ["open.ppt.skip_warning", "skip_ppt_warning", "boolean"],
      ["open.offline", "filter.load.offline", "boolean"],
      ["statusbar.icon", "statusbar.preferences", "boolean"],
      
      ["save.insert.link", "save.insertlink", "boolean"],
      ["save.insert.link.pos", "save.insertlink.pos", "string"],
      ["save.insert.link.date", "save.insertlink.date", "boolean"],
      ["save.insert.link.version", "save.insertlink.version", "boolean"],
      ["save.insert.link.nolocal", "save.insertlink.nolocal", "boolean"],
      ["save.insert.url", "save.inserturl", "boolean"],
      
      ["save.history.register", "save.register_download", "boolean"],
      ["save.progress.statusbar", "save.statusbar", "boolean"],
      ["save.show_errors", "save.show_errors", "boolean"],
      ["save.show_option_dialog", "save.show_option_dialog", "boolean"],
      
      ["save.local.same_dir", "save.local_to_same", "boolean"],
      
      ["save.single.type", "save.type", "string"],
      
      ["save.multi.type", "savemulti.type", "string"],
      ["save.multi.format", "savemulti.lastformat", "string"],
      ["save.multi.dir", "savemulti.base", "string"],
      ["save.multi.autoclose", "save.multi.autoclose", "boolean"],
      ["save.multi.check_none", "save.multi.none", "boolean"],
      
      ["save.quick.type", "quicksave.savetype", "string"],
      ["save.quick.dir", "quicksave.base", "string"],
      ["save.quick.dup.action", "quicksave.autoreplace.action", "string"],
      ["save.quick.use_selection", "quicksave.selection", "boolean"],
      
      ["save.mail.type", "sendmail.savetype", "string"],
      ["save.mail.app", "sendmail.app", "string"],
      ["save.mail.arg.other", "sendmail.arg", "string"],
      
      ["save.filename", "save.filename", "string"],
      
      ["save.filename.dup.rename.digits", "save.dup.digits", "number"],
      ["save.filename.dup.rename.init", "save.dup.init", "number"],
      
      ["save.filename.custom", "save.filename.custom", "string"],
      ["save.filename.custom.am", "save.filename.custom.am", "string"],
      ["save.filename.custom.pm", "save.filename.custom.pm", "string"],
      ["save.filename.custom.limit", "save.filename.custom.limit", "boolean"],
      ["save.filename.custom.limit.count", "save.filename.custom.limit.count", "number"],
      
      ["menu.context.save", "save.contextmenu", "boolean"],
      ["menu.context.save.quick", "quicksave.showmenu", "boolean"],
      ["menu.context.save.mail", "sendmail.showmenu", "boolean"],
      ["menu.tool", "toolmenu.preferences", "boolean"],
      
      ["shortcut.save.single", "save.shortcut", "boolean"],
      ["shortcut.save.single.keycode", "save.shortcut.keycode", "string"],
      ["shortcut.save.single.mods.alt", "save.shortcut.modifiers.alt", "boolean"],
      ["shortcut.save.single.mods.ctrl", "save.shortcut.modifiers.ctrl", "boolean"],
      ["shortcut.save.single.mods.meta", "save.shortcut.modifiers.meta", "boolean"],
      ["shortcut.save.single.mods.shift", "save.shortcut.modifiers.shift", "boolean"],
      
      ["shortcut.save.multi", "save.shortcut2", "boolean"],
      ["shortcut.save.multi.keycode", "save.shortcut2.keycode", "string"],
      ["shortcut.save.multi.mods.alt", "save.shortcut2.modifiers.alt", "boolean"],
      ["shortcut.save.multi.mods.ctrl", "save.shortcut2.modifiers.ctrl", "boolean"],
      ["shortcut.save.multi.mods.meta", "save.shortcut2.modifiers.meta", "boolean"],
      ["shortcut.save.multi.mods.shift", "save.shortcut2.modifiers.shift", "boolean"],
      
      ["shortcut.save.quick", "quicksave.shortcut", "boolean"],
      ["shortcut.save.quick.keycode", "quicksave.shortcut.keycode", "string"],
      ["shortcut.save.quick.mods.alt", "quicksave.shortcut.modifiers.alt", "boolean"],
      ["shortcut.save.quick.mods.ctrl", "quicksave.shortcut.modifiers.ctrl", "boolean"],
      ["shortcut.save.quick.mods.meta", "quicksave.shortcut.modifiers.meta", "boolean"],
      ["shortcut.save.quick.mods.shift", "quicksave.shortcut.modifiers.shift", "boolean"],
      
      ["convert.dir.suffix", "convert.dir_suffix", "string"],
      
      ["open.max_run_time", "nolimit", "boolean"],
      ["open.max_run_time.time", "nolimit.time", "number"],
      ["save.max_run_time", "save.nolimit", "boolean"],
      ["save.max_run_time.time", "save.nolimit.time", "number"],
      
      ["open.cache.expire", "cache_expire.enabled", "boolean"],
      ["open.cache.expire.time", "cache_expire", "number"],
      
      ["save.mail.debug", "sendmail.debug", "boolean"],
      
      ["use_global_lastdir", "use_global_lastdir", "boolean"],
      
      ["open.lastdir", "save.lastdir", "string"],
      ["save.lastdir", "save.lastdir", "string"]
    ];
    
    this._migrate_simple(oldPrefix, newPrefix, migrates, prefHandler);
    
    let oldName, newName, value;
    
    oldName = "sendmail.argtype";
    if (prefHandler.prefHas(oldPrefix + oldName)) {
      value = prefHandler.getCharPref(oldPrefix + oldName);
      if (value == "index") {
        value = "other";
      }
      
      newName = "save.mail.arg.type";
      prefHandler.setCharPref(newPrefix + newName, value);
    }
    
    oldName = "sendmail.app";
    if (prefHandler.prefHas(oldPrefix + oldName)) {
      value = prefHandler.getCharPref(oldPrefix + oldName);
      if (value != "") {
        value = "other";
      } else {
        value = defaultMail;
      }
      
      newName = "save.mail.app.type";
      prefHandler.setCharPref(newPrefix + newName, value);
    }
    
    oldName = "save.skip_alert_on_deep";
    if (prefHandler.prefHas(oldPrefix + oldName)) {
      value = prefHandler.getBoolPref(oldPrefix + oldName);
      if (!value) {
        value = "ask";
      } else {
        oldName = "save.skip_alert_on_deep.default";
        if (prefHandler.prefHas(oldPrefix + oldName)) {
          value = prefHandler.getCharPref(oldPrefix + oldName);
        } else {
          value = "ask";
        }
      }
      
      if (value) {
        newName = "save.deep_import.action";
        prefHandler.setCharPref(newPrefix + newName, value);
      }
    }
    
    oldName = "save.multi.autoreplace";
    if (prefHandler.prefHas(oldPrefix + oldName)) {
      value = prefHandler.getBoolPref(oldPrefix + oldName);
      if (!value) {
        value = "ask";
      } else {
        oldName = "save.multi.autoreplace.action";
        if (prefHandler.prefHas(oldPrefix + oldName)) {
          value = prefHandler.getCharPref(oldPrefix + oldName);
        } else {
          value = "ask";
        }
      }
      
      if (value) {
        newName = "save.multi.dup.action";
        prefHandler.setCharPref(newPrefix + newName, value);
      }
    }
    
    oldName = "save.local_to_same.append";
    if (prefHandler.prefHas(oldPrefix + oldName)) {
      value = prefHandler.getBoolPref(oldPrefix + oldName);
      if (!value) {
        value = "";
      } else {
        oldName = "save.local_to_same.append.text";
        if (prefHandler.prefHas(oldPrefix + oldName)) {
          value = prefHandler.getCharPref(oldPrefix + oldName);
        } else {
          value = "";
        }
      }
      
      newName = "save.local.same_dir.append";
      prefHandler.setCharPref(newPrefix + newName, value);
    }
  },
  
  /**
   * 6.0.0 から 6.1.0 へのマイグレート
   *
   * @param   {object} prefHandler
   *          設定操作関数セット
   */
  _migrate_6_0_0_to_6_1_0: function(prefHandler) {
    let oldPrefix = this.prefix;
    let newPrefix = this.prefix;
    
    let migrates = [
      ["save.multi.state", "save.multi.type", "string"],
      ["save.quick.state", "save.quick.type", "string"],
      ["save.mail.state",  "save.mail.type",  "string"]
    ];
    
    this._migrate_simple(oldPrefix, newPrefix, migrates, prefHandler);
    
    let oldName, newName, value;
    
    oldName = "save.single.type";
    if (prefHandler.prefHas(oldPrefix + oldName)) {
      let saveType = prefHandler.getCharPref(oldPrefix + oldName);
      
      let value = saveType.replace(/_add$/, "");
      
      newName = "save.single.state";
      prefHandler.setCharPref(newPrefix + newName, value);
      
      if (/_add$/.test(saveType)) {
        value = true;
      } else {
        value = false;
      }
      newName = "save.single.add";
      prefHandler.setBoolPref(newPrefix + newName, value);
    }
  },

  /**
   * 6.3.0 から 6.4.0 へのマイグレート
   *
   * @param   {object} prefHandler
   *          設定操作関数セット
   */
  _migrate_6_3_0_to_6_4_0: function(prefHandler) {
    let oldPrefix = this.prefix;
    let newPrefix = this.prefix;
    
    let migrates = [
      ["save.import.limit.exceeded.action", "save.deep_import.action", "string"]
    ];
    
    this._migrate_simple(oldPrefix, newPrefix, migrates, prefHandler);
  },
  
  /**
   * 1:1 の設定マイグレート
   *
   * @param   {string} oldPrefix
   *          古い設定のプレフィックス
   * @param   {string} newPrefix
   *          新しい設定のプレフィックス
   * @param   {Array.<[string,string,string]>} migrates
   *            [[string  新しい設定名, string  古い設定名,  string  型], ...]
   * @param   {object} prefHandler
   *          設定操作関数セット
   */
  _migrate_simple: function(oldPrefix, newPrefix, migrates, prefHandler) {
    migrates.forEach(function([newName, oldName, prefType]) {
      if (prefHandler.prefHas(oldPrefix + oldName)) {
        try {
          switch (prefType) {
            case "boolean": {
              let value = prefHandler.getBoolPref(oldPrefix + oldName);
              prefHandler.setBoolPref(newPrefix + newName, value);
              break;
            }
            case "string": {
              let value = prefHandler.getCharPref(oldPrefix + oldName);
              prefHandler.setCharPref(newPrefix + newName, unescape(value));
              break;
            }
            case "number": {
              let value = prefHandler.getIntPref(oldPrefix + oldName);
              prefHandler.setIntPref(newPrefix + newName, value);
              break;
            }
          }
        } catch (e) {
          Cu.reportError("Failed to migrate.\n"
                         + oldName + "\n"
                         + newName + "\n"
                         + prefType + "\n"
                         + e + "\n"
                         + e.stack);
        }
      }
    });
  }
};
arPrototype.extend(UnMHTPrefClass, arPrefClass);
let UnMHTPref = new UnMHTPrefClass();
UnMHTPref.migrate();
UnMHTPref.setPref("version", UnMHTVersion);
