"use strict";

let { classes: Cc, interfaces: Ci, results: Cr, utils: Cu } = Components;
Cu.import("resource://gre/modules/Services.jsm");
Cu.import("resource://gre/modules/XPCOMUtils.jsm");
XPCOMUtils.importRelative(this, "base/arTimer.jsm");
XPCOMUtils.importRelative(this, "UnMHTScheme.jsm");
XPCOMUtils.importRelative(this, "UnMHTWindowUtil.jsm");
XPCOMUtils.defineLazyGetter(this, "UnMHTFileLoader", function() {
  XPCOMUtils.importRelative(this, "UnMHTFileLoader.jsm");
  return UnMHTFileLoader;
});

let EXPORTED_SYMBOLS = ["UnMHTStartupExtract"];

/**
 * アプリケーション起動以外のスタートアップ時に
 * タブにある MHT ファイルを展開する
 *
 * @implements arIStartupEventListener
 */
let UnMHTStartupExtract = {
  /* ==== arIStartupEventListener インターフェース ==== */
  
  /**
   * スタートアップ時のイベント
   *
   * @param   {object} data
   *          Bootstrap データ
   * @param   {number} reason
   *          理由
   */
  onStartup: function(data, reason) {
    if (reason == 0) {
      Cu.unload("resource://unmht/modules/UnMHTStartupExtract.jsm");
      return;
    }
    
    arTimer.setTimeout(function() {
      this._reloadTabs();
    }.bind(this), 100);
  },
  
  /* ==== private ==== */
  
  /**
   * タブにある MHT ファイルを展開する
   */
  _reloadTabs: function() {
    for (let window of XPCOMUtils.IterSimpleEnumerator(
      Services.wm.getEnumerator("navigator:browser"), Ci.nsIDOMWindow)) {
      UnMHTWindowUtil.getDocuments(window)
        .forEach(function(targetDocument) {
          let location = targetDocument.location.href;
          if (!UnMHTScheme.isUnMHTURI(location)) {
            return;
          }
          
          let nav
            = targetDocument.defaultView
            .QueryInterface(Ci.nsIInterfaceRequestor)
            .getInterface(Ci.nsIWebNavigation);
          let param = UnMHTScheme.getUnMHTURIParam(location);
          
          if (param.scheme != "cache" &&
              param.scheme != "cid" &&
              param.scheme != "dummy" &&
              param.scheme != "empty" &&
              param.scheme != "error") {
            let uri;
            try {
              uri = Services.io.newURI(param.original, null, null);
            } catch (e) {
              return;
            }
            
            let loadParam = {
              window:         window,
              uri:            uri,
              targetWindow:   nav,
              hasDocument:    false,
              replaceHistory: true,
              extract:        true,
              refName:        param.refName,
              useCache:       false
            };
            UnMHTFileLoader.loadWithParam(loadParam);
          } else {
            nav.loadURI(location, nav.LOAD_FLAGS_REPLACE_HISTORY,
                        null, null, null);
          }
        }.bind(this));
    }
    
    Cu.unload("resource://unmht/modules/UnMHTStartupExtract.jsm");
  }
};
