"use strict";

let { classes: Cc, interfaces: Ci, results: Cr, utils: Cu } = Components;
Cu.import("resource://gre/modules/XPCOMUtils.jsm");
XPCOMUtils.importRelative(this, "base/arWindowHookLoader.jsm");
XPCOMUtils.importRelative(this, "UnMHT.jsm");
XPCOMUtils.importRelative(this, "UnMHTOverlay.jsm");
XPCOMUtils.importRelative(this, "UnMHTPref.jsm");
XPCOMUtils.defineLazyGetter(this, "UnMHTFileLoadHook", function() {
  XPCOMUtils.importRelative(this, "UnMHTFileLoadHook.jsm");
  return UnMHTFileLoadHook;
});
XPCOMUtils.defineLazyGetter(this, "UnMHTInfoPanelClass", function() {
  XPCOMUtils.importRelative(this, "UnMHTInfoPanel.jsm");
  return UnMHTInfoPanelClass;
});
XPCOMUtils.defineLazyGetter(this, "UnMHTKeyboardShortcut", function() {
  XPCOMUtils.importRelative(this, "UnMHTKeyboardShortcut.jsm");
  return UnMHTKeyboardShortcut;
});
XPCOMUtils.defineLazyGetter(this, "UnMHTMAFIntegration", function() {
  XPCOMUtils.importRelative(this, "UnMHTMAFIntegration.jsm");
  return UnMHTMAFIntegration;
});
XPCOMUtils.defineLazyGetter(this, "UnMHTMenuClass", function() {
  XPCOMUtils.importRelative(this, "UnMHTMenu.jsm");
  return UnMHTMenuClass;
});
XPCOMUtils.defineLazyGetter(this, "UnMHTPageInfo", function() {
  XPCOMUtils.importRelative(this, "UnMHTPageInfo.jsm");
  return UnMHTPageInfo;
});
XPCOMUtils.defineLazyGetter(this, "UnMHTScrapBookIntegration", function() {
  XPCOMUtils.importRelative(this, "UnMHTScrapBookIntegration.jsm");
  return UnMHTScrapBookIntegration;
});
XPCOMUtils.defineLazyGetter(this, "UnMHTStatusbarProgress", function() {
  XPCOMUtils.importRelative(this, "UnMHTStatusbarProgress.jsm");
  return UnMHTStatusbarProgress;
});
XPCOMUtils.defineLazyGetter(this, "UnMHTToolbarButtonClass", function() {
  XPCOMUtils.importRelative(this, "UnMHTToolbarButton.jsm");
  return UnMHTToolbarButtonClass;
});

let EXPORTED_SYMBOLS = ["UnMHTStartupWindowHook"];

/**
 * ウィンドウへの組み込み
 *
 * @implements arIStartupEventListener
 */
let UnMHTStartupWindowHook = {
  /* ==== arIStartupEventListener インターフェース ==== */
  
  /**
   * スタートアップ時のイベント
   *
   * @param   {object} data
   *          Bootstrap データ
   * @param   {number} reason
   *          理由
   */
  onStartup: function(data, reason) {
    let genBrowserHooks = function() {
      let hooks = [
        UnMHTMenuClass,
        UnMHTToolbarButtonClass,
        UnMHTInfoPanelClass,
        UnMHTKeyboardShortcut,
        UnMHTFileLoadHook,
        UnMHTStatusbarProgress
      ];
      if (UnMHTPref.getPref("integrataion.maf")) {
        hooks.push(UnMHTMAFIntegration);
      }
      if (UnMHTPref.getPref("integrataion.scrapbook")) {
        hooks.push(UnMHTScrapBookIntegration);
      }
      return hooks;
    };
    
    let genMailHooks = function() {
      let hooks = [
        UnMHTMenuClass,
        UnMHTToolbarButtonClass,
        UnMHTStatusbarProgress
      ];
      if (UnMHTPref.getPref("integrataion.maf")) {
        hooks.push(UnMHTMAFIntegration);
      }
      return hooks;
    };
    
    let browserOverlay = UnMHTOverlay.bind(null, false);
    let mailerOverlay = UnMHTOverlay.bind(null, true);

    let params = {
      "chrome://browser/content/browser.xul": {
        overlay: browserOverlay,
        props: {
          UnMHT: UnMHT
        },
        genHooks: genBrowserHooks
      },
      "chrome://navigator/content/navigator.xul": {
        overlay: browserOverlay,
        props: {
          UnMHT: UnMHT
        },
        genHooks: genBrowserHooks
      },
      "chrome://messenger/content/messenger.xul": {
        overlay: mailerOverlay,
        props: {
          UnMHT: UnMHT
        },
        genHooks: genMailHooks
      },
      "chrome://browser/content/pageinfo/pageInfo.xul": {
        genHooks: function() {
          return [
            UnMHTPageInfo
          ];
        }
      },
      "chrome://navigator/content/pageinfo/pageInfo.xul": {
        genHooks: function() {
          return [
            UnMHTPageInfo
          ];
        }
      }
    };

    let dialogs = [
      "chrome://unmht/content/pref/pref.xul",
      "chrome://unmht/content/additional.xul",
      "chrome://unmht/content/confirm.xul",
      "chrome://unmht/content/failedlist.xul",
      "chrome://unmht/content/save.xul",
      "chrome://unmht/content/savemulti.xul",
      "chrome://unmht/content/saveoptions.xul"
    ];
    
    let loader = new arWindowHookLoader(params, dialogs);
    loader.onStartup(data, reason);
  }
};
