"use strict";

let { classes: Cc, interfaces: Ci, results: Cr, utils: Cu } = Components;

let EXPORTED_SYMBOLS = ["UnMHTTreeScanner"];

/**
 * DOM ツリー走査
 *
 * @param   {nsIDOMElement} startNode
 *          開始要素
 */
function UnMHTTreeScanner(startNode) {
  this._gen = this._getGenerator(startNode);
}
UnMHTTreeScanner.prototype = {
  progress: 0,     /** 進捗状況 (0-1)
                    *  走査した HTMLElement の数の割合
                    *  @type {number} */
  isHead: false,   /** HEAD 要素内か
                    *  @type {boolean} */
  
  /**
   * 破棄
   */
  destruct: function() {
    if (this._gen) {
      this._gen.close(); 
      this._gen = null;
    }
  },
  
  /**
   * 次の要素を返す
   *
   * @returns {nsIDOMNode}
   *          次の要素
   */
  getNext: function() {
    return this._gen.next(); 
  },
  
  /* ==== private ==== */
  
  _gen: null,   /** DOM ツリー走査ジェネレータ
                 *  @type {Generator} */
  
  /**
   * 要素を返すジェネレータを作成する
   *
   * @param   {nsIDOMElement} startNode
   *          開始要素
   * @returns {Generator}
   *          要素を返すジェネレータ
   */
  _getGenerator: function(startNode) {
    let node = startNode;
    this.isHead = false;
    
    let i = 0;
    let nodeCount = 1;
    try {
      nodeCount = startNode.getElementsByTagName("*").length + 1;
    } catch (e) {
    }
    while (node) {
      if (node instanceof Ci.nsIDOMHTMLHeadElement) {
        this.isHead = true;
      }
      
      this.progress = i / nodeCount;
      
      yield node;
      if (node instanceof Ci.nsIDOMElement) {
        i ++;
      }
      
      if (node.firstChild) {
        node = node.firstChild;
      } else {
        while (node) {
          if (node.nextSibling) {
            if (node instanceof Ci.nsIDOMHTMLHeadElement) {
              this.isHead = false;
            }
            
            node = node.nextSibling;
            break;
          }
          node = node.parentNode;
          if (node == startNode) {
            this.progress = 1;
            yield false;
            return;
          }
        }
      }
    }
  }
};
