"use strict";

let { classes: Cc, interfaces: Ci, results: Cr, utils: Cu } = Components;
Cu.import("resource://gre/modules/Services.jsm");
Cu.import("resource://gre/modules/XPCOMUtils.jsm");

let EXPORTED_SYMBOLS = ["arAsyncProcs"];

/**
 * 順番に実行する非同期関数列
 */
function arAsyncProcs() {
  this._fs = [];
  this._onFails = [];
}
arAsyncProcs.prototype = {
  /**
   * 手続きを追加する
   *
   * @param   {Function} f
   *          実行する手続
   *            function(cont, fail, ...)
   *              *param   {Function} cont
   *                       次の手続きを実行する
   *              *param   {Function} fail
   *                       手続を中断する
   *              残りの引数は直前の cont の引数
   */
  next: function(f) {
    this._fs.push(f);
    
    return this;
  },
  
  /**
   * 失敗時の手続きを追加する
   *
   * @param   {Function} f
   *          失敗時に実行する手続
   *            function(...)
   *              引数は fail の引数
   */
  onFail: function(f) {
    this._onFails.push(f);
    
    return this;
  },
  
  /**
   * 実行する
   */
  run: function() {
    let fs = this._fs;
    let onFails = this._onFails;
    
    let i = 0;
    let fail = function(...args) {
      onFails.forEach(function(onFail) {
        onFail.apply(null, args);
      });
    };
    let cont = function(...args) {
      if (i >= fs.length) {
        return;
      }
      
      let f = fs[i];
      i ++;
      f.apply(null, [cont, fail, ...args]);
    };
    cont();
  },
  
  /* ==== private ==== */
  
  _fs: null,      /** 実行する手続き
                   *  @type {Array.<Function>} */
  _onFails: null  /** 失敗時に実行する手続き
                   *  @type {Array.<Function>} */
};
