"use strict";

let { classes: Cc, interfaces: Ci, results: Cr, utils: Cu } = Components;

let EXPORTED_SYMBOLS = ["arShutdownEvent"];

/**
 * シャットダウンイベント通知
 */
let arShutdownEvent = {
  /**
   * イベントリスナを追加する
   *
   * @param   {arIShutdownEventListener} listener
   *          イベントリスナ
   */
  addListener: function(listener) {
    this._listeners.push(listener);
  },
  
  /**
   * シャットダウンイベントを通知する
   *
   * @param   {object} data
   *          Bootstrap データ
   * @param   {number} reason
   *          理由
   */
  broadcast: function(data, reason) {
    this._listeners.forEach(function(listener) {
      try {
        listener.onShutdown(data, reason);
      } catch (e) {
        Cu.reportError(e + "\n"
                       + e.fileName + ":" + e.lineNumber + "\n"
                       + e.stack);
      }
    });
    this._listeners = [];
  },
  
  /* ==== private ==== */
  
  _listeners: [] /** イベントリスナ
                  *  @type {Array.<arIShutdownEventListener>} */
};
