"use strict";

let { classes: Cc, interfaces: Ci, results: Cr, utils: Cu } = Components;
Cu.import("resource://gre/modules/XPCOMUtils.jsm");
Cu.import("resource://gre/modules/Services.jsm");
XPCOMUtils.importRelative(this, "arShutdownEvent.jsm");

let EXPORTED_SYMBOLS = ["arStringBundle"];

/**
 * StringBundle 管理
 *
 * @implements arIShutdownEventListener
 */
function arStringBundle(uris = []) {
  arShutdownEvent.addListener(this);
  
  uris.forEach(this.loadBundle.bind(this));
}
arStringBundle.prototype = {
  /**
   * StringBundle を読み込む
   *
   * @param   {string} uri
   *          バンドルの URI
   */
  loadBundle: function(uri) {
    this._bundle = Services.strings.createBundle(uri);
  },
  
  /**
   * StringBundle から文字列を取得する
   *
   * @param   {string} name
   *          取得する文字列の名前
   * @param   {Array} params
   *          フォーマット用のパラメータ
   *          不要ならば undefined
   * @returns {string}
   *          StringBundle の文字列
   */
  get: function(name, params) {
    if (params) {
      return this._bundle.formatStringFromName(name, params, params.length);
    } else {
      return this._bundle.GetStringFromName(name);
    }
  },
  
  /**
   * キャッシュをクリアする
   */
  flushBundle: function() {
    Services.strings.flushBundles();
  },
  
  /* ==== arIShutdownEventListener ==== */
  
  /**
   * シャットダウンイベント
   *
   * @param   {object} data
   *          Bootstrap データ
   * @param   {number} reason
   *          理由
   */
  onShutdown: function(data, reason) {
    this.flushBundle();
  },
  
  /* ==== private ==== */

  _bundle: null /** StringBundle
                 *  @type {nsIStringBundle} */
};
