"use strict";

let { classes: Cc, interfaces: Ci, results: Cr, utils: Cu } = Components;
Cu.import("resource://gre/modules/Services.jsm");
Cu.import("resource://gre/modules/XPCOMUtils.jsm");
XPCOMUtils.importRelative(this, "arFileUtil.jsm");
XPCOMUtils.importRelative(this, "arShutdownEvent.jsm");

let EXPORTED_SYMBOLS = ["arSystemDirectory"];

/**
 * システムディレクトリ管理
 *
 * @param   {string} systemName
 *          システムディレクトリ名
 * @implements arIShutdownEventListener
 */
function arSystemDirectory(systemName) {
  this._systemName = systemName;
  
  arShutdownEvent.addListener(this);
}
arSystemDirectory.prototype = {
  /**
   * システムディレクトリを作成する
   */
  makeSystemDirectory: function() {
    this._systemDirectory = arFileUtil.profileDirectory.clone();
    this._systemDirectory.append(this._systemName);
    arFileUtil.createDirectory(this._systemDirectory);
  },
  
  /**
   * キャッシュディレクトリを作成する
   */
  makeCacheDirectory: function() {
    this._cacheDirectory = this._systemDirectory.clone();
    this._cacheDirectory.append("Cache");
    arFileUtil.createDirectory(this._cacheDirectory);
  },
  
  /**
   * キャッシュを削除する
   */
  clearCacheDirectory: function() {
    if (!this._cacheDirectory) {
      return;
    }
    
    try {
      if (!this._cacheDirectory.exists()) {
        return;
      }
      
      arFileUtil.getDirectoryEntries(this._cacheDirectory)
        .forEach(function(file) {
          file.remove(true);
        });
    } catch (e) {
    }
  },
  
  /* ---- ディレクトリ取得 ---- */
  
  /**
   * システムディレクトリ
   * @type {nsIFile}
   */
  get systemDirectory() this._systemDirectory,
  
  /**
   * キャッシュディレクトリ
   * @type {nsIFile}
   */
  get cacheDirectory() this._cacheDirectory,

  /* ==== arIShutdownEventListener ==== */
  
  /**
   * シャットダウンイベント
   *
   * @param   {object} data
   *          Bootstrap データ
   * @param   {number} reason
   *          理由
   */
  onShutdown: function(data, reason) {
    this.clearCacheDirectory();
  },
  
  /* ==== private ==== */
  
  _systemName: "",        /** システムディレクトリ名
                           *  @type {string} */
  
  _systemDirectory: null, /** システムディレクトリ
                           *  @type {nsIFile}
                           *  {Profile ディレクトリ}/{_systemName} */
  _cacheDirectory: null   /** キャッシュディレクトリ
                           *  @type {nsIFile}
                           *  {Profile ディレクトリ}/{_systemName}/Cache */
};
