"use strict";

let { classes: Cc, interfaces: Ci, results: Cr, utils: Cu } = Components;
Cu.import("resource://gre/modules/XPCOMUtils.jsm");

let EXPORTED_SYMBOLS = ["arTimer"];

/**
 * タイマー
 */
let arTimer = {
  /**
   * 指定時間後に関数を実行する
   *
   * @param   {Function} func
   *          実行する関数
   * @param   {number} timeout
   *          待ち時間 [ms]
   * @returns {nsITimer}
   *          タイマー
   */
  setTimeout: function(func, timeout) {
    let timer = Cc["@mozilla.org/timer;1"].createInstance(Ci.nsITimer);
    
    let callback = {
      QueryInterface: XPCOMUtils.generateQI([Ci.nsITimerCallback]),
      
      /* ==== nsITimerCallback インターフェース ==== */
      
      /**
       * タイマーイベント
       *
       * @param   {nsITimer} timer
       *          対象のタイマー
       */
      notify: function(timer) {
        try {
          func();
        } catch (e) {
          try {
            Cu.reportError(e + "\n" + e.stack);
          } catch (e2) {
            /* アンロードされた後 */
            //throw e;
          }
        }
        this._timer = null;
      },
      
      /* ==== private ==== */
      
      _timer: timer, /** 対象のタイマー
                      *  @type {nsITimer}
                      *    notify が呼ばれるまでは保持しておかないと
                      *    タイマーが GC の対象になってしまう */
    };
    
    timer.initWithCallback(callback, timeout, Ci.nsITimer.TYPE_ONE_SHOT);
    
    return timer;
  }
};
